/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.builders.client.render;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.lwjgl.opengl.GL11;

import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.ChunkPos;
import net.minecraft.util.math.Vec3d;

import buildcraft.lib.chunkload.ChunkLoaderManager;
import buildcraft.lib.client.render.DetachedRenderer;
import buildcraft.lib.debug.DebugRenderHelper;

import buildcraft.builders.tile.TileQuarry;

public class AdvDebuggerQuarry implements DetachedRenderer.IDetachedRenderer {
    private static final int COLOUR_CHUNK = 0x55_99_FF_99;

    private final WeakReference<TileQuarry> tileReference;

    public AdvDebuggerQuarry(TileQuarry tile) {
        tileReference = new WeakReference<>(tile);
    }

    @Override
    public void render(EntityPlayer player, float partialTicks) {
        TileQuarry tile = tileReference.get();
        if (tile == null || !tile.frameBox.isInitialized()) {
            return;
        }
        List<ChunkPos> chunkPoses = new ArrayList<>(ChunkLoaderManager.getChunksToLoad(tile));
        chunkPoses.sort(
            Comparator.comparingDouble(chunkPos ->
                -player.func_174824_e(partialTicks).func_72438_d(
                    new Vec3d(
                        chunkPos.func_180334_c() + 0.5 + (chunkPos.func_180332_e() - chunkPos.func_180334_c()) / 2,
                        player.func_174824_e(partialTicks).field_72448_b,
                        chunkPos.func_180333_d() + 0.5 + (chunkPos.func_180330_f() - chunkPos.func_180333_d()) / 2
                    )
                )
            )
        );
        GlStateManager.func_179147_l();
        BufferBuilder bb = Tessellator.func_178181_a().func_178180_c();
        bb.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176600_a);
        for (ChunkPos chunkPos : chunkPoses) {
            DebugRenderHelper.renderAABB(
                bb,
                new AxisAlignedBB(
                    chunkPos.func_180334_c() + 0.5D,
                    tile.frameBox.min().func_177956_o() + 0.5D,
                    chunkPos.func_180333_d() + 0.5D,
                    chunkPos.func_180332_e() + 0.5D,
                    tile.frameBox.max().func_177956_o() + 0.5D,
                    chunkPos.func_180330_f() + 0.5D
                ),
                COLOUR_CHUNK
            );
        }
        Tessellator.func_178181_a().func_78381_a();
        GlStateManager.func_179084_k();
    }
}
